'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var ramda = require('ramda');
var PropTypes = _interopDefault(require('prop-types'));
var reactRedux = require('react-redux');
var namespaces = require('@redux-tools/namespaces');
var ramdaExtension = require('ramda-extension');
var utilsReact = require('@redux-tools/utils-react');
var hoistNonReactStatics = _interopDefault(require('hoist-non-react-statics'));
var invariant = _interopDefault(require('invariant'));
var injectors = require('@redux-tools/injectors');

var NamespaceContext = React.createContext({
  namespaces: {},
  useNamespace: ramdaExtension.alwaysNull
});

var mergeContextValues = ramda.mergeDeepWith(ramda.flip(ramda.or));

var Provider = function Provider(_ref) {
  var children = _ref.children,
      feature = _ref.feature,
      namespace = _ref.namespace,
      store = _ref.store,
      useNamespace = _ref.useNamespace;
  var context = React.useContext(NamespaceContext);
  var nextContext = React.useMemo(function () {
    var _namespaces;

    return mergeContextValues(context, {
      // NOTE: Defaulting here is safer than using `Provider.defaultProps`,
      // because passing `null` would not result in a fallback to `DEFAULT_FEATURE`.
      namespaces: (_namespaces = {}, _namespaces[feature || namespaces.DEFAULT_FEATURE] = namespace, _namespaces),
      useNamespace: useNamespace
    });
  }, [context, feature, namespace, useNamespace]);
  var providerElement = React__default.createElement(NamespaceContext.Provider, {
    value: nextContext
  }, children);
  return store ? React__default.createElement(reactRedux.Provider, {
    store: store
  }, providerElement) : providerElement;
};

Provider.propTypes = {
  children: PropTypes.node.isRequired,
  feature: PropTypes.string,
  namespace: PropTypes.string,
  store: PropTypes.object,
  useNamespace: PropTypes.func
};

var useNamespace = function useNamespace(feature) {
  var _useContext = React.useContext(NamespaceContext),
      _useContext$namespace = _useContext.namespaces,
      namespaces$1 = _useContext$namespace === void 0 ? {} : _useContext$namespace,
      _useContext$useNamesp = _useContext.useNamespace,
      useNamespace = _useContext$useNamesp === void 0 ? ramdaExtension.alwaysNull : _useContext$useNamesp;

  var namespace = useNamespace(feature || namespaces.DEFAULT_FEATURE, namespaces$1);
  return namespaces$1[feature] || namespace || namespaces$1[namespaces.DEFAULT_FEATURE];
};

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var IS_SERVER = Boolean(typeof process !== 'undefined' && process && process.versions && process.versions.node);

var useUniversalLayoutEffect = IS_SERVER ? React.useEffect : React.useLayoutEffect;
var getOtherProps = ramda.omit(['isGlobal', 'global', 'isPersistent', 'persist']);

var makeHook = function makeHook(config) {
  invariant(ramdaExtension.isObject(config), 'The injector config is undefined.');
  var getEject = config.getEject,
      getEntries = config.getEntries,
      getInject = config.getInject,
      ejectMethodName = config.ejectMethodName,
      injectMethodName = config.injectMethodName,
      type = config.type;
  var pascalCaseType = ramdaExtension.toPascalCase(type);
  var hookName = "use" + pascalCaseType;

  var useInjectables = function useInjectables(injectables, options) {
    if (options === void 0) {
      options = {};
    }

    var injectableKeys = ramda.keys(injectables);
    var locationMessage = "@redux-tools: This warning happened while injecting the following " + type + ": " + (injectableKeys + ".");

    var warn = function warn() {
      var _console;

      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      return (_console = console).warn.apply(_console, [locationMessage].concat(args));
    }; // NOTE: `options.global` and `options.persist` are deprecated.


    var isGlobal = options.isGlobal || options.global || false;
    var isPersistent = options.isPersistent || options.persist || false;
    var feature = options.feature || namespaces.DEFAULT_FEATURE;
    var contextNamespace = useNamespace(feature);

    var _useContext = React.useContext(reactRedux.ReactReduxContext),
        store = _useContext.store;

    var namespace = isGlobal ? null : options.namespace || contextNamespace;
    var inject = getInject(store);
    var eject = getEject(store); // NOTE: On the server, the injectables should be injected beforehand.

    var _useState = React.useState(IS_SERVER),
        isInitialized = _useState[0],
        setIsInitialized = _useState[1];

    var props = ramdaExtension.rejectNil(_extends({}, getOtherProps(options), {
      feature: feature,
      namespace: namespace
    })); // TODO: Refactor when React DevTools support multiple debug values or non-primitive structures.

    React.useDebugValue(String(["Namespace: " + namespace, "Feature: " + feature, pascalCaseType + ": " + injectableKeys, "Initialized: " + isInitialized]));

    if (IS_SERVER) {
      var areEntriesAlreadyInjected = ramda.all(function (entry) {
        return ramda.includes(entry, getEntries(store));
      }, injectors.createEntries(injectables, props));

      if (!areEntriesAlreadyInjected) {
        warn("When rendering on the server, inject all " + type + " before calling", "'ReactDOMServer.renderToString()'. You should do this inside an", "'async getInitialProps()' function, i.e. where you fetch data and", 'do other side effects. If you need to do server-side injections', 'during rendering, open an issue.');
      }
    } // NOTE: This doesn't run on the server, but won't trigger `useLayoutEffect` warnings either.


    useUniversalLayoutEffect(function () {
      if (isGlobal && feature !== namespaces.DEFAULT_FEATURE) {
        warn("You are using a feature (" + feature + ") with global " + type + ".", 'This will have no effect.');
      }

      if (!namespace && !isGlobal) {
        warn("You're injecting " + type + " with no namespace!", 'They will be injected globally. If this is intended, consider passing', "'isGlobal: true' to the injector, e.g. '" + hookName + "(" + type + ", { isGlobal: true })'.");
      }

      if (ramdaExtension.isNotNil(options.global)) {
        warn("'global: " + options.global + "' is deprecated. Use 'isGlobal: " + options.global + "'.");
      }

      if (ramdaExtension.isNotNil(options.persist)) {
        warn("'persist: " + options.persist + "' is deprecated. Use 'isPersistent: " + options.persist + "'.");
      }

      invariant(inject, "'store." + injectMethodName + "' is missing. Are you using the enhancer correctly?");
      invariant(eject, "'store." + ejectMethodName + "' is missing. Are you using the enhancer correctly?");
      inject(injectables, props);
      setIsInitialized(true);
      return function () {
        if (!isPersistent) {
          eject(injectables, props);
        }
      };
    }, []);
    return isInitialized;
  };

  return useInjectables;
};

var makeDecorator = function makeDecorator(config) {
  invariant(ramdaExtension.isObject(config), 'The injector config is undefined.');
  var useInjectables = makeHook(config);
  var type = config.type;
  var decoratorName = type ? "With" + ramdaExtension.toPascalCase(type) : 'Injector';
  return function (injectables, options) {
    if (options === void 0) {
      options = {};
    }

    return function (NextComponent) {
      var Injector = function Injector(_ref) {
        var propFeature = _ref.feature,
            propNamespace = _ref.namespace,
            otherProps = _objectWithoutPropertiesLoose(_ref, ["feature", "namespace"]);

        var feature = options.feature || propFeature || namespaces.DEFAULT_FEATURE;
        var contextNamespace = useNamespace(feature);
        var namespace = options.namespace || propNamespace || contextNamespace;
        var isInitialized = useInjectables(injectables, _extends({}, options, {
          feature: feature,
          namespace: namespace
        }));

        if (isInitialized) {
          return React__default.createElement(NextComponent, _extends({
            feature: feature,
            key: String([feature, namespace]),
            namespace: namespace
          }, otherProps));
        }

        return null;
      };

      Injector.propTypes = {
        feature: PropTypes.string,
        namespace: PropTypes.string
      };
      hoistNonReactStatics(Injector, NextComponent);
      Injector.displayName = decoratorName + "(" + utilsReact.getDisplayName(NextComponent) + ")";
      return Injector;
    };
  };
};

exports.Provider = Provider;
exports.makeDecorator = makeDecorator;
exports.makeHook = makeHook;
exports.useNamespace = useNamespace;
